@@ -136,6 +136,7 @@ typedef enum {
 	ACTION_CHANGED_SIGNAL,
 	DUMMY_SUCCESS_SIGNAL,
 	EJECT_FAILURE_SIGNAL,
+	INSTALL_MISSING_SIGNAL,
 	LAST_SIGNAL
 } BraseroBurnSignalType;
 
@@ -228,14 +229,14 @@ brasero_burn_action_changed_real (BraseroBurn *burn,
 		               0,
 		               1.0,
 		               1.0,
-		               -1);
+		               -1L);
 	else if (action == BRASERO_BURN_ACTION_EJECTING)
 		g_signal_emit (burn,
 			       brasero_burn_signals [PROGRESS_CHANGED_SIGNAL],
 			       0,
 			       -1.0,
 			       -1.0,
-			       -1);
+			       -1L);
 }
 
 static gboolean
@@ -1345,7 +1346,7 @@ start:
 				       0,
 				       1.0,
 				       1.0,
-				       -1);
+				       -1L);
 		}
 		return BRASERO_BURN_OK;
 	}
@@ -1407,7 +1408,8 @@ start:
 		goto start;
 	}
 	else if (error_code == BRASERO_BURN_ERROR_DISK_SPACE
-	     ||  error_code == BRASERO_BURN_ERROR_PERMISSION) {
+	     ||  error_code == BRASERO_BURN_ERROR_PERMISSION
+	     ||  error_code == BRASERO_BURN_ERROR_TMP_DIRECTORY) {
 		gboolean is_temp;
 
 		/* That's an imager (outputs an image to the disc) so that means
@@ -1518,7 +1520,7 @@ start:
 			       0,
 			       1.0,
 			       1.0,
-			       -1);
+			       -1L);
 		return BRASERO_BURN_OK;
 	}
 
@@ -1659,6 +1661,69 @@ start:
 	return BRASERO_BURN_ERR;
 }
 
+static BraseroBurnResult
+brasero_burn_install_missing (BraseroPluginErrorType error,
+			      const gchar *details,
+			      gpointer user_data)
+{
+	GValue instance_and_params [3];
+	GValue return_value;
+
+	instance_and_params [0].g_type = 0;
+	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (user_data));
+	g_value_set_instance (instance_and_params, user_data);
+	
+	instance_and_params [1].g_type = 0;
+	g_value_init (instance_and_params + 1, G_TYPE_INT);
+	g_value_set_int (instance_and_params + 1, error);
+
+	instance_and_params [2].g_type = 0;
+	g_value_init (instance_and_params + 2, G_TYPE_STRING);
+	g_value_set_string (instance_and_params + 2, details);
+
+	return_value.g_type = 0;
+	g_value_init (&return_value, G_TYPE_INT);
+	g_value_set_int (&return_value, BRASERO_BURN_ERROR);
+
+	g_signal_emitv (instance_and_params,
+			brasero_burn_signals [INSTALL_MISSING_SIGNAL],
+			0,
+			&return_value);
+
+	g_value_unset (instance_and_params);
+
+	return g_value_get_int (&return_value);		       
+}
+
+static BraseroBurnResult
+brasero_burn_list_missing (BraseroPluginErrorType type,
+			   const gchar *detail,
+			   gpointer user_data)
+{
+	GString *string = user_data;
+
+	if (type == BRASERO_PLUGIN_ERROR_MISSING_APP ||
+	    type == BRASERO_PLUGIN_ERROR_SYMBOLIC_LINK_APP ||
+	    type == BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing application */
+		g_string_append_printf (string, _("%s (application)"), detail);
+	}
+	else if (type == BRASERO_PLUGIN_ERROR_MISSING_LIBRARY ||
+	         type == BRASERO_PLUGIN_ERROR_LIBRARY_VERSION) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing library */
+		g_string_append_printf (string, _("%s (library)"), detail);
+	}
+	else if (type == BRASERO_PLUGIN_ERROR_MISSING_GSTREAMER_PLUGIN) {
+		g_string_append_c (string, '\n');
+		/* Translators: %s is the name of a missing GStreamer plugin */
+		g_string_append_printf (string, _("%s (GStreamer plugin)"), detail);
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_burn_check_session_consistency (BraseroBurn *burn,
                                         BraseroTrackType *output,
@@ -1772,6 +1837,42 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 
 	brasero_burn_session_set_flags (priv->session, retval);
 	BRASERO_BURN_LOG_FLAGS (retval, "Flags after checking =");
+
+	/* Check missing applications/GStreamer plugins.
+	 * This is the best place. */
+	brasero_burn_session_set_strict_support (BRASERO_BURN_SESSION (priv->session), TRUE);
+	result = brasero_burn_session_can_burn (priv->session, FALSE);
+	brasero_burn_session_set_strict_support (BRASERO_BURN_SESSION (priv->session), FALSE);
+
+	if (result == BRASERO_BURN_OK)
+		return result;
+
+	result = brasero_burn_session_can_burn (priv->session, FALSE);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	result = brasero_session_foreach_plugin_error (priv->session,
+	                                               brasero_burn_install_missing,
+	                                               burn);
+	if (result != BRASERO_BURN_OK) {
+		if (result != BRASERO_BURN_CANCEL) {
+			GString *string;
+
+			string = g_string_new (_("Please install the following required applications and libraries manually and try again:"));
+			brasero_session_foreach_plugin_error (priv->session,
+			                                      brasero_burn_list_missing,
+	        			                      string);
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_MISSING_APP_AND_PLUGIN,
+				     string->str);
+
+			g_string_free (string, TRUE);
+		}
+
+		return BRASERO_BURN_ERR;
+	}
+
 	return BRASERO_BURN_OK;
 }
 
@@ -1800,35 +1901,15 @@ brasero_burn_check_data_loss (BraseroBurn *burn,
 	input = brasero_track_type_new ();
 	brasero_burn_session_get_input_type (priv->session, input);
 
-	if (flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
-		/* There is an error if APPEND was set since this disc is not
-		 * supported without a prior blanking. */
-		
-		/* we warn the user is going to lose data even if in the case of
-		 * DVD+/-RW we don't really blank the disc we rather overwrite */
-		result = brasero_burn_emit_signal (burn,
-						   WARN_DATA_LOSS_SIGNAL,
-						   BRASERO_BURN_CANCEL);
-		if (result == BRASERO_BURN_NEED_RELOAD)
-			goto reload;
+	if (media & (BRASERO_MEDIUM_HAS_DATA|BRASERO_MEDIUM_HAS_AUDIO)) {
+		if (flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE) {
+			/* There is an error if APPEND was set since this disc is not
+			 * supported without a prior blanking. */
 
-		if (result != BRASERO_BURN_OK) {
-			brasero_track_type_free (input);
-			return result;
-		}
-	}
-	else if (media & (BRASERO_MEDIUM_HAS_DATA|BRASERO_MEDIUM_HAS_AUDIO)) {
-		/* A few special warnings for the discs with data/audio on them
-		 * that don't need prior blanking or can't be blanked */
-		if ((media & BRASERO_MEDIUM_CD)
-		&&  brasero_track_type_get_has_stream (input)
-		&& !BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (input))) {
-			/* We'd rather blank and rewrite a disc rather than
-			 * append audio to appendable disc. That's because audio
-			 * tracks have little chance to be readable by common CD
-			 * player as last tracks */
+			/* we warn the user is going to lose data even if in the case of
+			 * DVD+/-RW we don't really blank the disc we rather overwrite */
 			result = brasero_burn_emit_signal (burn,
-							   WARN_AUDIO_TO_APPENDABLE_SIGNAL,
+							   WARN_DATA_LOSS_SIGNAL,
 							   BRASERO_BURN_CANCEL);
 			if (result == BRASERO_BURN_NEED_RELOAD)
 				goto reload;
@@ -1838,37 +1919,59 @@ brasero_burn_check_data_loss (BraseroBurn *burn,
 				return result;
 			}
 		}
+		else {
+			/* A few special warnings for the discs with data/audio on them
+			 * that don't need prior blanking or can't be blanked */
+			if ((media & BRASERO_MEDIUM_CD)
+			&&  brasero_track_type_get_has_stream (input)
+			&& !BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (input))) {
+				/* We'd rather blank and rewrite a disc rather than
+				 * append audio to appendable disc. That's because audio
+				 * tracks have little chance to be readable by common CD
+				 * player as last tracks */
+				result = brasero_burn_emit_signal (burn,
+								   WARN_AUDIO_TO_APPENDABLE_SIGNAL,
+								   BRASERO_BURN_CANCEL);
+				if (result == BRASERO_BURN_NEED_RELOAD)
+					goto reload;
 
-		/* NOTE: if input is AUDIO we don't care since the OS
-		 * will load the last session of DATA anyway */
-		if ((media & BRASERO_MEDIUM_HAS_DATA)
-		&&   brasero_track_type_get_has_data (input)
-		&& !(flags & BRASERO_BURN_FLAG_MERGE)) {
-			/* warn the users that their previous data
-			 * session (s) will not be mounted by default by
-			 * the OS and that it'll be invisible */
-			result = brasero_burn_emit_signal (burn,
-							   WARN_PREVIOUS_SESSION_LOSS_SIGNAL,
-							   BRASERO_BURN_CANCEL);
-
-			if (result == BRASERO_BURN_RETRY) {
-				/* Wipe out the current flags,
-				 * Add a new one 
-				 * Recheck the result */
-				brasero_burn_session_pop_settings (priv->session);
-				brasero_burn_session_push_settings (priv->session);
-				brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_MERGE);
-				result = brasero_burn_check_session_consistency (burn, NULL, error);
-				if (result != BRASERO_BURN_OK)
+				if (result != BRASERO_BURN_OK) {
+					brasero_track_type_free (input);
 					return result;
+				}
 			}
 
-			if (result == BRASERO_BURN_NEED_RELOAD)
-				goto reload;
+			/* NOTE: if input is AUDIO we don't care since the OS
+			 * will load the last session of DATA anyway */
+			if ((media & BRASERO_MEDIUM_HAS_DATA)
+			&&   brasero_track_type_get_has_data (input)
+			&& !(flags & BRASERO_BURN_FLAG_MERGE)) {
+				/* warn the users that their previous data
+				 * session (s) will not be mounted by default by
+				 * the OS and that it'll be invisible */
+				result = brasero_burn_emit_signal (burn,
+								   WARN_PREVIOUS_SESSION_LOSS_SIGNAL,
+								   BRASERO_BURN_CANCEL);
 
-			if (result != BRASERO_BURN_OK) {
-				brasero_track_type_free (input);
-				return result;
+				if (result == BRASERO_BURN_RETRY) {
+					/* Wipe out the current flags,
+					 * Add a new one 
+					 * Recheck the result */
+					brasero_burn_session_pop_settings (priv->session);
+					brasero_burn_session_push_settings (priv->session);
+					brasero_burn_session_add_flag (priv->session, BRASERO_BURN_FLAG_MERGE);
+					result = brasero_burn_check_session_consistency (burn, NULL, error);
+					if (result != BRASERO_BURN_OK)
+						return result;
+				}
+
+				if (result == BRASERO_BURN_NEED_RELOAD)
+					goto reload;
+
+				if (result != BRASERO_BURN_OK) {
+					brasero_track_type_free (input);
+					return result;
+				}
 			}
 		}
 	}
@@ -2027,9 +2130,13 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 
 				if (session_sec > medium_sec) {
 					BRASERO_BURN_LOG ("Not enough space on medium %"G_GOFFSET_FORMAT"/%"G_GOFFSET_FORMAT, session_sec, medium_sec);
-					result = brasero_burn_reload_dest_media (burn,  BRASERO_BURN_ERROR_MEDIUM_SPACE, error);
-					if (result != BRASERO_BURN_OK)
-						break;
+					result = brasero_burn_reload_dest_media (burn,
+					                                         BRASERO_BURN_ERROR_MEDIUM_SPACE,
+					                                         error);
+					if (result == BRASERO_BURN_OK)
+						result = BRASERO_BURN_RETRY;
+
+					break;
 				}
 			}
 			brasero_track_type_free (type);
@@ -2194,7 +2301,7 @@ brasero_burn_check_real (BraseroBurn *self,
 			       0,
 			       1.0,
 			       1.0,
-			       -1);
+			       -1L);
 
 		g_object_unref (priv->task);
 		priv->task = NULL;
@@ -3059,6 +3166,17 @@ brasero_burn_class_init (BraseroBurnClass *klass)
 			      brasero_marshal_INT__OBJECT,
 			      G_TYPE_INT, 1,
 		              BRASERO_TYPE_DRIVE);
+        brasero_burn_signals [INSTALL_MISSING_SIGNAL] =
+		g_signal_new ("install_missing",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BraseroBurnClass,
+					       install_missing),
+			      NULL, NULL,
+			      brasero_marshal_INT__INT_STRING,
+			      G_TYPE_INT, 2,
+		              G_TYPE_INT,
+			      G_TYPE_STRING);
 }
 
 static void
