@@ -534,7 +534,6 @@ brasero_session_cfg_save_drive_properties (BraseroSessionCfg *self,
 {
 	BraseroSessionCfgPrivate *priv;
 	GConfClient *client;
-	const gchar *path;
 	guint64 rate;
 	gchar *key;
 
@@ -554,12 +553,6 @@ brasero_session_cfg_save_drive_properties (BraseroSessionCfg *self,
 	gconf_client_set_int (client, key, rate / 1000, NULL);
 	g_free (key);
 
-	/* temporary directory */
-	path = brasero_burn_session_get_tmpdir (BRASERO_BURN_SESSION (self));
-	key = g_strdup_printf ("%s/tmpdir", BRASERO_DRIVE_PROPERTIES_KEY);
-	gconf_client_set_string (client, key, path, NULL);
-	g_free (key);
-
 	g_object_unref (client);
 }
 
@@ -1051,7 +1044,7 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 	/* Make sure the session is ready */
 	status = brasero_status_new ();
 	result = brasero_burn_session_get_status (BRASERO_BURN_SESSION (self), status);
-	if (result == BRASERO_BURN_NOT_READY) {
+	if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING) {
 		g_object_unref (status);
 
 		priv->is_valid = BRASERO_SESSION_NOT_READY;
@@ -1156,8 +1149,9 @@ brasero_session_cfg_update (BraseroSessionCfg *self)
 	/* Check that current input and output work */
 	if (brasero_track_type_get_has_stream (source)) {
 		if (priv->CD_TEXT_modified) {
-			/* Try to redo what we undid (after all a new plugin could have
-			 * been activated in the mean time ...) and see what happens */
+			/* Try to redo what we undid (after all a new plugin
+			 * could have been activated in the mean time ...) and
+			 * see what happens */
 			brasero_track_type_set_stream_format (source,
 							      BRASERO_METADATA_INFO|
 							      brasero_track_type_get_stream_format (source));
