@@ -33,16 +33,18 @@
 #endif
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <fcntl.h>
 
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
+#include <gconf/gconf-client.h>
+
 #include "brasero-session.h"
 #include "brasero-session-helper.h"
 
@@ -54,6 +56,7 @@
 
 #include "brasero-medium.h"
 #include "brasero-drive.h"
+#include "brasero-drive-priv.h"
 #include "brasero-medium-monitor.h"
 
 #include "brasero-tags.h"
@@ -90,16 +93,15 @@ struct _BraseroSessionSetting {
 	gchar *label;
 	guint64 rate;
 
-	gchar *tmpdir;
-
 	BraseroBurnFlag flags;
 };
 typedef struct _BraseroSessionSetting BraseroSessionSetting;
 
 struct _BraseroBurnSessionPrivate {
-	FILE *session;
+	int session;
 	gchar *session_path;
 
+	gchar *tmpdir;
 	GSList *tmpfiles;
 
 	BraseroSessionSetting settings [1];
@@ -136,6 +138,8 @@ typedef enum {
 static guint brasero_burn_session_signals [LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
+#define  BRASERO_TEMPORARY_DIRECTORY_KEY    "/apps/brasero/drives/tmpdir"
+
 static void
 brasero_session_settings_clean (BraseroSessionSetting *settings)
 {
@@ -145,9 +149,6 @@ brasero_session_settings_clean (BraseroSessionSetting *settings)
 	if (settings->toc)
 		g_free (settings->toc);
 
-	if (settings->tmpdir)
-		g_free (settings->tmpdir);
-
 	if (settings->label)
 		g_free (settings->label);
 
@@ -169,7 +170,6 @@ brasero_session_settings_copy (BraseroSessionSetting *dest,
 	dest->image = g_strdup (original->image);
 	dest->toc = g_strdup (original->toc);
 	dest->label = g_strdup (original->label);
-	dest->tmpdir = g_strdup (original->tmpdir);
 }
 
 static void
@@ -513,8 +513,8 @@ brasero_burn_session_get_status (BraseroBurnSession *session,
 	BraseroBurnSessionPrivate *priv;
 	BraseroStatus *track_status;
 	gdouble num_tracks = 0.0;
-	gdouble done = -1.0;
 	guint not_ready = 0;
+	gdouble done = -1.0;
 	GSList *iter;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (session), BRASERO_TRACK_TYPE_NONE);
@@ -524,6 +524,13 @@ brasero_burn_session_get_status (BraseroBurnSession *session,
 		return BRASERO_BURN_ERR;
 
 	track_status = brasero_status_new ();
+
+	if (priv->settings->burner && brasero_drive_probing (priv->settings->burner)) {
+		BRASERO_BURN_LOG ("Drive not ready yet");
+		brasero_status_set_not_ready (status, -1, NULL);
+		return BRASERO_BURN_NOT_READY;
+	}
+
 	for (iter = priv->tracks; iter; iter = iter->next) {
 		BraseroTrack *track;
 		BraseroBurnResult result;
@@ -532,7 +539,7 @@ brasero_burn_session_get_status (BraseroBurnSession *session,
 		result = brasero_track_get_status (track, track_status);
 		num_tracks ++;
 
-		if (result == BRASERO_BURN_NOT_READY)
+		if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING)
 			not_ready ++;
 		else if (result != BRASERO_BURN_OK) {
 			g_object_unref (track_status);
@@ -1177,22 +1184,26 @@ brasero_burn_session_set_tmpdir (BraseroBurnSession *self,
 				 const gchar *path)
 {
 	BraseroBurnSessionPrivate *priv;
+	GConfClient *client;
 
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
-	if (priv->settings->tmpdir && path
-	&& !strcmp (priv->settings->tmpdir, path))
+	if (!g_strcmp0 (priv->tmpdir, path))
 		return BRASERO_BURN_OK;
 
-	if (priv->settings->tmpdir)
-		g_free (priv->settings->tmpdir);
+	if (priv->tmpdir)
+		g_free (priv->tmpdir);
 
 	if (path)
-		priv->settings->tmpdir = g_strdup (path);
+		priv->tmpdir = g_strdup (path);
 	else
-		priv->settings->tmpdir = NULL;
+		priv->tmpdir = NULL;
+
+	client = gconf_client_get_default ();
+	gconf_client_set_string (client, BRASERO_TEMPORARY_DIRECTORY_KEY, priv->tmpdir, NULL);
+	g_object_unref (client);
 
 	return BRASERO_BURN_OK;
 }
@@ -1214,7 +1225,7 @@ brasero_burn_session_get_tmpdir (BraseroBurnSession *self)
 	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), NULL);
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-	return priv->settings->tmpdir? priv->settings->tmpdir:g_get_tmp_dir ();
+	return priv->tmpdir? priv->tmpdir:g_get_tmp_dir ();
 }
 
 /**
@@ -1246,8 +1257,8 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	/* create a working directory in tmp */
-	tmpdir = priv->settings->tmpdir ?
-		 priv->settings->tmpdir :
+	tmpdir = priv->tmpdir ?
+		 priv->tmpdir :
 		 g_get_tmp_dir ();
 
 	tmp = g_build_path (G_DIR_SEPARATOR_S,
@@ -1264,13 +1275,13 @@ brasero_burn_session_get_tmp_dir (BraseroBurnSession *self,
 		if (errsv != EACCES)
 			g_set_error (error, 
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
+				     BRASERO_BURN_ERROR_TMP_DIRECTORY,
 				     "%s",
 				     g_strerror (errsv));
 		else
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_PERMISSION,
+				     BRASERO_BURN_ERROR_TMP_DIRECTORY,
 				     _("You do not have the required permission to write at this location"));
 		return BRASERO_BURN_ERR;
 	}
@@ -1318,8 +1329,8 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 		return BRASERO_BURN_OK;
 
 	/* takes care of the output file */
-	tmpdir = priv->settings->tmpdir ?
-		 priv->settings->tmpdir :
+	tmpdir = priv->tmpdir ?
+		 priv->tmpdir :
 		 g_get_tmp_dir ();
 
 	name = g_strconcat (BRASERO_BURN_TMP_FILE_NAME, suffix, NULL);
@@ -1339,13 +1350,13 @@ brasero_burn_session_get_tmp_file (BraseroBurnSession *self,
 		if (errsv != EACCES)
 			g_set_error (error, 
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
+				     BRASERO_BURN_ERROR_TMP_DIRECTORY,
 				     "%s",
 				     g_strerror (errsv));
 		else
 			g_set_error (error, 
 				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_PERMISSION,
+				     BRASERO_BURN_ERROR_TMP_DIRECTORY,
 				     _("You do not have the required permission to write at this location"));
 
 		return BRASERO_BURN_ERR;
@@ -2123,6 +2134,7 @@ brasero_burn_session_logv (BraseroBurnSession *self,
 			   const gchar *format,
 			   va_list arg_list)
 {
+	int len;
 	gchar *message;
 	gchar *offending;
 	BraseroBurnSessionPrivate *priv;
@@ -2143,12 +2155,13 @@ brasero_burn_session_logv (BraseroBurnSession *self,
 	if (!g_utf8_validate (message, -1, (const gchar**) &offending))
 		*offending = '\0';
 
-	if (fwrite (message, strlen (message), 1, priv->session) != 1)
+	len = strlen (message);
+	if (write (priv->session, message, len) != len)
 		g_warning ("Some log data couldn't be written: %s\n", message);
 
 	g_free (message);
 
-	if (fwrite ("\n", 1, 1, priv->session) != 1)
+	if (write (priv->session, "\n", 1) != 1)
 		g_warning ("Some log data could not be written");
 }
 
@@ -2169,24 +2182,6 @@ brasero_burn_session_log (BraseroBurnSession *self,
 	va_end (args);
 }
 
-void
-brasero_burn_session_set_log_path (BraseroBurnSession *self,
-				   const gchar *session_path)
-{
-	BraseroBurnSessionPrivate *priv;
-
-	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-	if (priv->session_path) {
-		g_free (priv->session_path);
-		priv->session_path = NULL;
-	}
-
-	if (session_path)
-		priv->session_path = g_strdup (session_path);
-}
-
 const gchar *
 brasero_burn_session_get_log_path (BraseroBurnSession *self)
 {
@@ -2208,28 +2203,31 @@ brasero_burn_session_start (BraseroBurnSession *self)
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
-	if (!priv->session_path) {
-		int fd;
-		const gchar *tmpdir;
+	/* This must obey the path of the temporary directory if possible */
+	priv->session_path = g_build_path (G_DIR_SEPARATOR_S,
+					   priv->tmpdir,
+					   BRASERO_BURN_TMP_FILE_NAME,
+					   NULL);
+	priv->session = g_mkstemp_full (priv->session_path,
+	                                O_CREAT|O_WRONLY,
+	                                S_IRWXU);
 
-		/* takes care of the output file */
-		tmpdir = priv->settings->tmpdir ?
-			 priv->settings->tmpdir :
-			 g_get_tmp_dir ();
+	if (priv->session < 0) {
+		g_free (priv->session_path);
 
-		/* This must obey the path of the temporary directory */
 		priv->session_path = g_build_path (G_DIR_SEPARATOR_S,
-						   tmpdir,
+						   g_get_tmp_dir (),
 						   BRASERO_BURN_TMP_FILE_NAME,
 						   NULL);
-
-		fd = g_mkstemp (priv->session_path);
-		priv->session = fdopen (fd, "w");
+		priv->session = g_mkstemp_full (priv->session_path,
+		                                O_CREAT|O_WRONLY,
+		                                S_IRWXU);
 	}
-	else
-		priv->session = fopen (priv->session_path, "w");
 
-	if (!priv->session) {
+	if (priv->session < 0) {
+		g_free (priv->session_path);
+		priv->session_path = NULL;
+
 		g_warning ("Impossible to open a session file\n");
 		return FALSE;
 	}
@@ -2304,16 +2302,17 @@ brasero_burn_session_stop (BraseroBurnSession *self)
 	g_return_if_fail (BRASERO_IS_BURN_SESSION (self));
 
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-	if (priv->session) {
-		fclose (priv->session);
-		priv->session = NULL;
+	if (priv->session > 0) {
+		close (priv->session);
+		priv->session = -1;
+	}
+
+	if (priv->session_path) {
+		g_free (priv->session_path);
+		priv->session_path = NULL;
 	}
 }
 
-/**
- *
- */
-
 static void
 brasero_burn_session_track_list_free (GSList *list)
 {
@@ -2387,6 +2386,7 @@ static void
 brasero_burn_session_finalize (GObject *object)
 {
 	BraseroBurnSessionPrivate *priv;
+	GConfClient *client;
 	GSList *iter;
 
 	BRASERO_BURN_LOG ("Cleaning session");
@@ -2437,6 +2437,15 @@ brasero_burn_session_finalize (GObject *object)
 		priv->pile_settings = NULL;
 	}
 
+	client = gconf_client_get_default ();
+	gconf_client_set_string (client, BRASERO_TEMPORARY_DIRECTORY_KEY, priv->tmpdir, NULL);
+	g_object_unref (client);
+
+	if (priv->tmpdir) {
+		g_free (priv->tmpdir);
+		priv->tmpdir = NULL;
+	}
+
 	/* clean tmpfiles */
 	for (iter = priv->tmpfiles; iter; iter = iter->next) {
 		gchar *tmpfile;
@@ -2448,9 +2457,9 @@ brasero_burn_session_finalize (GObject *object)
 	}
 	g_slist_free (priv->tmpfiles);
 
-	if (priv->session) {
-		fclose (priv->session);
-		priv->session = NULL;
+	if (priv->session > 0) {
+		close (priv->session);
+		priv->session = -1;
 	}
 
 	if (priv->session_path) {
@@ -2466,7 +2475,17 @@ brasero_burn_session_finalize (GObject *object)
 
 static void
 brasero_burn_session_init (BraseroBurnSession *obj)
-{ }
+{
+	BraseroBurnSessionPrivate *priv;
+	GConfClient *client;
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (obj);
+	priv->session = -1;
+
+	client = gconf_client_get_default ();
+	priv->tmpdir = gconf_client_get_string (client, BRASERO_TEMPORARY_DIRECTORY_KEY, NULL);
+	g_object_unref (client);
+}
 
 static void
 brasero_burn_session_class_init (BraseroBurnSessionClass *klass)
