@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /***************************************************************************
  *            audio-disc.c
  *
@@ -43,6 +45,8 @@
 #include "brasero-io.h"
 #include "brasero-notify.h"
 
+#include "brasero-units.h"
+
 #include "brasero-tags.h"
 #include "brasero-track-stream-cfg.h"
 #include "brasero-session-cfg.h"
@@ -155,7 +159,7 @@ static GtkActionEntry entries[] = {
 	 G_CALLBACK (brasero_audio_disc_edit_information_cb)},
 	{"DeleteAudio", GTK_STOCK_REMOVE, NULL, NULL, N_("Remove the selected files from the project"),
 	 G_CALLBACK (brasero_audio_disc_delete_activated_cb)},
-	{"PasteAudio", GTK_STOCK_PASTE, NULL, NULL, N_("Add the files stored in the clipboard"),
+	{"PasteAudio", NULL, N_("Paste files"), NULL, N_("Add the files stored in the clipboard"),
 	 G_CALLBACK (brasero_audio_disc_paste_activated_cb)},
 	{"Pause", "insert-pause", N_("I_nsert a Pause"), NULL, N_("Add a 2 second pause after the track"),
 	 G_CALLBACK (brasero_audio_disc_add_pause_cb)},
@@ -217,8 +221,6 @@ enum {
 	PROP_REJECT_FILE,
 };
 
-/* 1 sec = 75 sectors, len is in nanosecond */
-#define BRASERO_SECTORS_TO_TIME(sectors)	(gint64) (sectors * GST_SECOND / 75)
 #define COL_KEY "column_key"
 
 #define BRASERO_AUDIO_DISC_CONTEXT		1000
@@ -236,6 +238,9 @@ brasero_audio_disc_is_empty (BraseroDisc *disc)
 	GtkTreeModel *model;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (BRASERO_AUDIO_DISC (disc)->priv->tree));
+	if (!model)
+		return TRUE;
+
 	return gtk_tree_model_iter_n_children (model, NULL) != 0;
 }
 
@@ -413,12 +418,12 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 							   "text", BRASERO_VIDEO_TREE_MODEL_INDEX,
 							   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree), column);
-	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_column_set_resizable (column, TRUE);
 
 	/* Other columns */
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_view_column_set_min_width (column, 200);
+	gtk_tree_view_column_set_expand (column, TRUE);
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
@@ -451,14 +456,10 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 					    "editable", BRASERO_VIDEO_TREE_MODEL_EDITABLE);
 	gtk_tree_view_column_set_title (column, _("Title"));
 	g_object_set (G_OBJECT (column),
-		      "expand", TRUE,
 		      "spacing", 4,
 		      NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree),
-				     column);
-
-	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (obj->priv->tree),
-					   column);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree), column);
+	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (obj->priv->tree), column);
 
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set_data (G_OBJECT (renderer), COL_KEY, BRASERO_TRACK_STREAM_ARTIST_TAG);
@@ -478,8 +479,7 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 	column = gtk_tree_view_column_new_with_attributes (_("Artist"), renderer,
 							   "text", BRASERO_VIDEO_TREE_MODEL_ARTIST,
 							   NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree),
-				     column);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree), column);
 	gtk_tree_view_column_set_resizable (column, TRUE);
 	gtk_tree_view_column_set_min_width (column, 200);
 
@@ -488,9 +488,8 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 							   "text", BRASERO_VIDEO_TREE_MODEL_SIZE,
 							  /* "background", BACKGROUND_COL, */
 							   NULL);
-	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree),
-				     column);
-	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (obj->priv->tree), column);
+	gtk_tree_view_column_set_resizable (column, TRUE);
 
 	scroll = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_show (scroll);
@@ -530,7 +529,7 @@ brasero_audio_disc_reset_real (BraseroAudioDisc *disc)
 	}
 
 	if (disc->priv->message)
-		brasero_notify_message_remove (BRASERO_NOTIFY (disc->priv->message), BRASERO_AUDIO_DISC_CONTEXT);
+		brasero_notify_message_remove (disc->priv->message, BRASERO_AUDIO_DISC_CONTEXT);
 }
 
 static void
@@ -541,8 +540,8 @@ brasero_audio_disc_finalize (GObject *object)
 	
 	brasero_audio_disc_reset_real (cobj);
 
-	g_free (cobj->priv->add_dir);
-	g_free (cobj->priv->add_playlist);
+	brasero_io_job_base_free (cobj->priv->add_dir);
+	brasero_io_job_base_free (cobj->priv->add_playlist);
 	cobj->priv->add_dir = NULL;
 	cobj->priv->add_playlist = NULL;
 
@@ -614,7 +613,7 @@ brasero_audio_disc_add_uri_real (BraseroAudioDisc *disc,
 	brasero_track_stream_set_boundaries (BRASERO_TRACK_STREAM (track),
 					     start,
 					     end,
-					     BRASERO_SECTORS_TO_TIME (gap_sectors));
+					     BRASERO_SECTORS_TO_DURATION (gap_sectors));
 
 	session = brasero_video_tree_model_get_session (BRASERO_VIDEO_TREE_MODEL (store));
 	if (pos > 0) {
@@ -801,9 +800,13 @@ brasero_audio_disc_unreadable_dialog (BraseroAudioDisc *disc,
 				      GError *error)
 {
 	gchar *primary;
-	gchar *name = NULL;
+	gchar *name;
+	GFile *file;
+
+	file = g_file_new_for_uri (uri);
+	name = g_file_get_basename (file);
+	g_object_unref (file);
 
-	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
 	primary = g_strdup_printf (_("\"%s\" could not be opened."), name);
 	brasero_app_alert (brasero_app_get_default (),
 			   primary,
@@ -837,7 +840,7 @@ brasero_audio_disc_wav_dts_file_dialog (BraseroAudioDisc *disc)
 	BraseroSessionCfg *session;
 	GtkTreeModel *model;
 
-	if (brasero_notify_get_message_by_context_id (BRASERO_NOTIFY (disc->priv->message),
+	if (brasero_notify_get_message_by_context_id (disc->priv->message,
 	                                              BRASERO_AUDIO_DISC_CONTEXT))
 		return;
 
@@ -848,7 +851,7 @@ brasero_audio_disc_wav_dts_file_dialog (BraseroAudioDisc *disc)
 	                                  BRASERO_SESSION_STREAM_AUDIO_FORMAT,
 	                                  BRASERO_AUDIO_FORMAT_RAW);
 
-	message = brasero_notify_message_add (BRASERO_NOTIFY (disc->priv->message),
+	message = brasero_notify_message_add (disc->priv->message,
 					      _("Do you want to create an audio CD with DTS tracks?"),
 					      _("Some of the selected songs are suitable for creating DTS tracks."
 					        "\nThis type of audio CD track provides a higher quality of sound but can only be played by specific digital players."
@@ -856,20 +859,17 @@ brasero_audio_disc_wav_dts_file_dialog (BraseroAudioDisc *disc)
 					      0,
 					      BRASERO_AUDIO_DISC_CONTEXT);
 
-	brasero_disc_message_set_image (BRASERO_DISC_MESSAGE (message),
-					GTK_MESSAGE_INFO);
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (message), GTK_MESSAGE_INFO);
 
-	brasero_notify_button_add (BRASERO_NOTIFY (disc->priv->message),
-				   BRASERO_DISC_MESSAGE (message),
-				   _("Create _Regular Tracks"),
-				   _("Click here to burn all songs as regular tracks"),
-				   GTK_RESPONSE_NO);
+	gtk_widget_set_tooltip_text (gtk_info_bar_add_button (GTK_INFO_BAR (message),
+							    						  _("Create _Regular Tracks"),
+							    						  GTK_RESPONSE_NO),
+					     	     _("Click here to burn all songs as regular tracks"));
 
-	brasero_notify_button_add (BRASERO_NOTIFY (disc->priv->message),
-				   BRASERO_DISC_MESSAGE (message),
-				   _("Create _DTS Tracks"),
-				   _("Click here to burn all suitable songs as DTS tracks"),
-				   GTK_RESPONSE_OK);
+	gtk_widget_set_tooltip_text (gtk_info_bar_add_button (GTK_INFO_BAR (message),
+							    						  _("Create _DTS Tracks"),
+							    						  GTK_RESPONSE_OK),
+					     	     _("Click here to burn all suitable songs as DTS tracks"));
 
 	g_signal_connect (BRASERO_DISC_MESSAGE (message),
 			  "response",
@@ -887,7 +887,7 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 	BraseroStatus *status;
 	gboolean should_use_dts;
 
-	if (!GTK_WIDGET (self)->window)
+	if (!gtk_widget_get_window (GTK_WIDGET (self)))
 		return;
 
 	/* make sure all tracks have video */
@@ -949,7 +949,7 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 			continue;
 		}
 
-		if (result == BRASERO_BURN_NOT_READY) {
+		if (result == BRASERO_BURN_NOT_READY || result == BRASERO_BURN_RUNNING) {
 			notready = TRUE;
 			continue;
 		}
@@ -984,7 +984,7 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 			g_free (uri);
 		}
 	}
-	brasero_status_free (status);
+	g_object_unref (status);
 
 	if (should_use_dts
 	&&  brasero_burn_session_tag_lookup (BRASERO_BURN_SESSION (session), BRASERO_SESSION_STREAM_AUDIO_FORMAT, NULL) != BRASERO_BURN_OK)
@@ -1606,8 +1606,8 @@ brasero_audio_disc_edit_single_song_properties (BraseroAudioDisc *disc,
         /* information about the track */
         gtk_tree_model_get_iter (model, &iter, treepath);
         gtk_tree_model_get (model, &iter,
-                                         BRASERO_VIDEO_TREE_MODEL_INDEX_NUM, &track_num,
-                                         -1);
+                            BRASERO_VIDEO_TREE_MODEL_INDEX_NUM, &track_num,
+                            -1);
         brasero_track_stream_get_length (BRASERO_TRACK_STREAM (track), &length);
 
 	/* set up dialog */
@@ -1648,9 +1648,9 @@ brasero_audio_disc_edit_single_song_properties (BraseroAudioDisc *disc,
 					   &gap);
 
 	brasero_track_stream_set_boundaries (BRASERO_TRACK_STREAM (track),
-                                                                      start,
-                                                                      end,
-                                                                      BRASERO_SECTORS_TO_TIME (gap));
+                                             start,
+                                             end,
+                                             gap);
 
 	if (title)
 		brasero_track_tag_add_string (BRASERO_TRACK (track),
@@ -1672,7 +1672,7 @@ brasero_audio_disc_edit_single_song_properties (BraseroAudioDisc *disc,
 					   BRASERO_TRACK_STREAM_ISRC_TAG,
 					   isrc);
 
-	if (end - start + BRASERO_SECTORS_TO_TIME (gap) < BRASERO_MIN_STREAM_LENGTH)
+	if (end - start + BRASERO_SECTORS_TO_DURATION (gap) < BRASERO_MIN_STREAM_LENGTH)
 		brasero_audio_disc_short_track_dialog (disc);
 
 	g_free (title);
@@ -1825,8 +1825,14 @@ brasero_audio_disc_button_pressed_cb (GtkTreeView *tree,
 {
 	GtkWidgetClass *widget_class;
 
+	/* Avoid minding signals that happen out of the tree area (like in the 
+	 * headers for example) */
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (tree)))
+		return FALSE;
+
 	widget_class = GTK_WIDGET_GET_CLASS (tree);
 
+	/* Check that the click happened in the main window with rows. */
 	if (event->button == 3) {
 		GtkTreeSelection *selection;
 		GtkTreePath *path = NULL;
@@ -1890,7 +1896,7 @@ brasero_audio_disc_button_pressed_cb (GtkTreeView *tree,
 		 * so it can update itself (paticularly its selection) before we have
 		 * a look at it */
 		widget_class->button_press_event (GTK_WIDGET (tree), event);
-		
+
 		if (!treepath) {
 			GtkTreeSelection *selection;
 
@@ -1901,7 +1907,7 @@ brasero_audio_disc_button_pressed_cb (GtkTreeView *tree,
 			gtk_tree_selection_unselect_all (selection);
 			return FALSE;
 		}
-	
+
 		if (!result)
 			return FALSE;
 
