@@ -532,6 +532,8 @@ brasero_utils_disc_hide_use_info_data_received_cb (GtkWidget *widget,
 {
 	GtkWidget *other_widget;
 
+	g_return_if_fail(BRASERO_IS_PROJECT(project));
+
 	other_widget = brasero_utils_disc_find_tree_view (project->priv->current);
 	if (!other_widget)
 		return;
@@ -1366,12 +1368,15 @@ brasero_project_list_missing (BraseroPluginErrorType type,
 {
 	GString *string = user_data;
 
-	if (type == BRASERO_PLUGIN_ERROR_MISSING_APP) {
+	if (type == BRASERO_PLUGIN_ERROR_MISSING_APP ||
+	    type == BRASERO_PLUGIN_ERROR_SYMBOLIC_LINK_APP ||
+	    type == BRASERO_PLUGIN_ERROR_WRONG_APP_VERSION) {
 		g_string_append_c (string, '\n');
 		/* Translators: %s is the name of a missing application */
 		g_string_append_printf (string, _("%s (application)"), detail);
 	}
-	else if (type == BRASERO_PLUGIN_ERROR_MISSING_LIBRARY) {
+	else if (type == BRASERO_PLUGIN_ERROR_MISSING_LIBRARY ||
+	         type == BRASERO_PLUGIN_ERROR_LIBRARY_VERSION) {
 		g_string_append_c (string, '\n');
 		/* Translators: %s is the name of a missing library */
 		g_string_append_printf (string, _("%s (library)"), detail);
@@ -1458,6 +1463,14 @@ brasero_project_setup_session (BraseroProject *project,
 	}
 }
 
+static void
+brasero_project_output_changed (BraseroBurnSession *session,
+                                BraseroMedium *former_medium,
+                                GtkDialog *dialog)
+{
+	gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
+}
+
 static BraseroBurnResult
 brasero_project_drive_properties (BraseroProject *project)
 {
@@ -1469,6 +1482,7 @@ brasero_project_drive_properties (BraseroProject *project)
 	GtkWidget *options;
 	GtkWidget *button;
 	GtkWidget *dialog;
+	glong cancel_sig;
 	GtkWidget *box;
 	gchar *header;
 	gchar *string;
@@ -1489,6 +1503,12 @@ brasero_project_drive_properties (BraseroProject *project)
 					      NULL);
 	g_free (header);
 
+	/* This is in case the medium gets ejected instead of our locking it */
+	cancel_sig = g_signal_connect (project->priv->session,
+	                               "output-changed",
+	                               G_CALLBACK (brasero_project_output_changed),
+	                               dialog);
+
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       _("Burn _Several Copies"),
 			       GTK_RESPONSE_ACCEPT);
@@ -1533,6 +1553,8 @@ brasero_project_drive_properties (BraseroProject *project)
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 
+	g_signal_handler_disconnect (project->priv->session, cancel_sig);
+
 	if (answer == GTK_RESPONSE_OK)
 		return BRASERO_BURN_OK;
 
@@ -1625,7 +1647,6 @@ brasero_project_burn (BraseroProject *project)
 	project->priv->current = NULL;
 
 	brasero_dest_selection_set_session (BRASERO_DEST_SELECTION (project->priv->selection), NULL);
-
 	brasero_project_setup_session (project, BRASERO_BURN_SESSION (project->priv->session));
 
 	/* This is to stop the preview widget from playing */
@@ -1641,11 +1662,12 @@ brasero_project_burn (BraseroProject *project)
 
 	project->priv->current = current_disc;
 	brasero_disc_set_session_contents (current_disc, BRASERO_BURN_SESSION (project->priv->session));
-
 	brasero_dest_selection_set_session (BRASERO_DEST_SELECTION (project->priv->selection),
 					    BRASERO_BURN_SESSION (project->priv->session));
 
 	project->priv->is_burning = 0;
+
+	brasero_project_update_controls (project);
 }
 
 /******************************** cover ****************************************/
